#
# Conditional build:
%bcond_with	lto	# enable link time optimisations (breaks tests)
%bcond_without	static	# static version of mksh
%bcond_without	tests	# rtchecks and test.sh checks

%define		vernr	%(echo %{version} | tr -d 'a-z')

Summary:	MirBSD Korn Shell
Summary(pl.UTF-8):	Powłoka Korna z MirBSD
Name:		mksh
Version:	59c
Release:	3
License:	BSD
Group:		Applications/Shells
Source0:	https://www.mirbsd.org/MirOS/dist/mir/mksh/%{name}-R%{version}.tgz
# Source0-md5:	99f8ac3c1d8a30b913d509f1969a4aaa
Source1:	%{name}-mkshrc
Source2:	get-source.sh
Patch0:		%{name}-mkshrc_support.patch
Patch1:		%{name}-circumflex.patch
Patch2:         drop-builtins.patch
URL:		https://www.mirbsd.org/mksh.htm
%if %{with tests}
BuildRequires:	ed
BuildRequires:	perl-base
%endif
%{?with_static:BuildRequires:   glibc-static}
BuildRequires:	rpmbuild(macros) >= 1.670
# is needed for /etc directory existence
Requires(pre):	FHS
Requires:	setup >= 2.4.6-2
Obsoletes:	pdksh
BuildRoot:	%{tmpdir}/%{name}-%{version}-root-%(id -u -n)

%define		_bindir			/bin

%description
mksh is the MirBSD enhanced version of the Public Domain Korn shell
(pdksh), a Bourne-compatible shell which is largely similar to the
original AT&T Korn shell. It includes bug fixes and feature
improvements in order to produce a modern, robust shell good for
interactive and especially script use. It has UTF-8 support in the
emacs command line editing mode; corresponds to OpenBSD 4.2-current
ksh sans GNU bash-like $PS1; the build environment requirements are
autoconfigured; throughout code simplification/bugfix/enhancement has
been done, and the shell has extended compatibility to other modern
shells.

%description -l pl.UTF-8
mksh to pochodząca z MirBSD rozszerzona wersja powłoki Public Domain
Korn Shell (pdksh) - kompatybilnej z powłoką Bourne'a, w większości
zbliżonej do oryginalnej powłoki Korna z AT&T. Zawiera poprawki błędów
i rozszerzenia mające na celu stworzenie współczesnej powłoki o
bogatych możliwościach do użytku interaktywnego i (zwłaszcza) w
skryptach. Ma obsługę UTF-8 w trybie edycji linii poleceń w stylu
emacsa; $PS1 odpowiada temu z ksh obecnym w OpenBSD 4.2-current;
środowisko budowania jest automatycznie konfigurowane; dzięki
wykonanym uproszczeniom kodu, poprawkom i rozszerzeniom powłoka ma
rozszerzoną kompatybilność z innymi współczesnymi powłokami.

%package static
Summary:	Statically linked the MirBSD enhanced version of pdksh
Summary(pl.UTF-8):	Skonsolidowana statycznie powłoka mksh
Group:		Applications/Shells
# requires base for /etc/mkshrc?
Requires:	%{name} = %{version}-%{release}

%description static
mksh is the MirBSD enhanced version of the Public Domain Korn shell
(pdksh), a Bourne-compatible shell which is largely similar to the
original AT&T Korn shell.

This packege contains statically linked version of mksh.

%description static -l pl.UTF-8
mksh to pochodząca z MirBSD rozszerzona wersja powłoki Public Domain
Korn Shell (pdksh) - kompatybilnej z powłoką Bourne'a, w większości
zbliżonej do oryginalnej powłoki Korna z AT&T.

W tym pakiecie jest mksh skonsolidowany statycznie.

%prep
%setup -qc
%{__mv} mksh/* .; rmdir mksh

%patch0 -p1
%patch1 -p1
%patch2 -p1

sed -i -e 's|fgrep|grep -F|g' check.t

# sed rules instead of patch (needed update for every release)
sed -i -e 's|\(#define.*MKSH_VERSION.*\)"|\1 @DISTRO@"|g' sh.h
sed -i -e 's|\(KSH R%{vernr}.*\)|\1 @DISTRO@|g' check.t

# fill distro
sed -i -e 's#@DISTRO@#PLD/Linux 3.0#g' check.t sh.h
# sanity checks
grep PLD/Linux check.t || exit 1
grep PLD/Linux sh.h || exit 1

# we'll need this later due to -DMKSH_GCC55009
cat >rtchecks <<'EOF'
typeset -i sari=0
typeset -Ui uari=0
typeset -i x=0
print -r -- $((x++)):$sari=$uari.
let --sari --uari
print -r -- $((x++)):$sari=$uari.
sari=2147483647 uari=2147483647
print -r -- $((x++)):$sari=$uari.
let ++sari ++uari
print -r -- $((x++)):$sari=$uari.
let --sari --uari
let 'sari *= 2' 'uari *= 2'
let ++sari ++uari
print -r -- $((x++)):$sari=$uari.
let ++sari ++uari
print -r -- $((x++)):$sari=$uari.
sari=-2147483648 uari=-2147483648
print -r -- $((x++)):$sari=$uari.
let --sari --uari
print -r -- $((x++)):$sari=$uari.
EOF

cat >rtchecks.expected <<'EOF'
0:0=0.
1:-1=4294967295.
2:2147483647=2147483647.
3:-2147483648=2147483648.
4:-1=4294967295.
5:0=0.
6:-2147483648=2147483648.
7:2147483647=2147483647.
EOF

%build
install -d out

CC="%{__cc}" \
CFLAGS="%{rpmcflags} -DMKSH_GCC55009" \
LDFLAGS="%{rpmldflags}" \
CPPFLAGS="%{rpmcppflags}" \
sh ./Build.sh -Q -r -j %{?with_lto:-c lto}

# skip some tests if not on terminal
if ! tty -s; then
	skip_tests="-C regress:no-ctty"
fi

%if %{with tests}
./mksh rtchecks >rtchecks.got 2>&1
if ! cmp --quiet rtchecks.got rtchecks.expected ; then
	echo "rtchecks failed"
	diff -Naurp rtchecks.expected rtchecks.got
	exit 1
fi
./test.sh -v $skip_tests
%endif
mv mksh out/mksh.dynamic

%if %{with static}
CC="%{__cc}" \
CFLAGS="%{rpmcflags} -DMKSH_GCC55009" \
LDFLAGS="%{rpmldflags} -static" \
CPPFLAGS="%{rpmcppflags}" \
sh ./Build.sh -Q -r -j %{?with_lto:-c lto}

%if %{with tests}
./test.sh -v $skip_tests
./mksh rtchecks >rtchecks.got 2>&1
if ! cmp --quiet rtchecks.got rtchecks.expected ; then
	echo "rtchecks failed"
	diff -Naurp %{SOURCE3} rtchecks.got
	exit 1
fi
%endif
mv mksh out/mksh.static
%endif

%install
rm -rf $RPM_BUILD_ROOT
install -d $RPM_BUILD_ROOT{%{_bindir},%{_mandir}/man1}
install -p out/mksh.dynamic $RPM_BUILD_ROOT%{_bindir}/mksh
%{?with_static:install -p out/mksh.static $RPM_BUILD_ROOT%{_bindir}/mksh.static}

cp -a mksh.1 $RPM_BUILD_ROOT%{_mandir}/man1/mksh.1
echo ".so mksh.1" > $RPM_BUILD_ROOT%{_mandir}/man1/sh.1

install -D %{SOURCE1} $RPM_BUILD_ROOT%{_sysconfdir}/mkshrc
ln -sf mksh $RPM_BUILD_ROOT%{_bindir}/sh

# some pdksh scripts used that
ln -sf mksh $RPM_BUILD_ROOT%{_bindir}/ksh

%clean
rm -rf $RPM_BUILD_ROOT

%post   -p %add_etc_shells -p /bin/sh /bin/ksh /bin/mksh
%preun  -p %remove_etc_shells -p /bin/sh /bin/ksh /bin/mksh

%post	static -p %add_etc_shells -p /bin/mksh.static
%preun	static -p %remove_etc_shells -p /bin/mksh.static

%triggerpostun -p <lua> -- pdksh
if arg[2] ~= 0 then
%lua_add_etc_shells /bin/sh /bin/ksh /bin/mksh
end

%files
%defattr(644,root,root,755)
%doc dot.mkshrc
%config(noreplace) %verify(not md5 mtime size) %{_sysconfdir}/mkshrc
%attr(755,root,root) %{_bindir}/mksh
%attr(755,root,root) %{_bindir}/ksh
%attr(755,root,root) %{_bindir}/sh
%{_mandir}/man1/mksh.1*
%{_mandir}/man1/sh.1*

%if %{with static}
%files static
%defattr(644,root,root,755)
%attr(755,root,root) %{_bindir}/mksh.static
%endif
%changelog
* Thu Oct 05 2023 PLD Linux Team <feedback@pld-linux.org>
- For complete changelog see: http://git.pld-linux.org/?p=packages/mksh.git;a=log;h=master

* Thu Oct 05 2023 Arkadiusz Miśkiewicz <arekm@maven.pl> e920f87
- rel 3; drop builtin cat and sleep

* Tue Sep 14 2021 Arkadiusz Miśkiewicz <arekm@maven.pl> 2aacd9a
- Release 2 (by relup.sh)

* Mon Mar 22 2021 Arkadiusz Miśkiewicz <arekm@maven.pl> 6da35db
- Missing is not ok here.

* Sat Mar 20 2021 Arkadiusz Miśkiewicz <arekm@maven.pl> ee1bb74
- Rediff patches.

* Sun Nov 01 2020 Jan Palus <atler@pld-linux.org> 5683a35
- up to 59c

* Mon May 18 2020 Jan Palus <atler@pld-linux.org> 57ffb32
- up to 59b

* Wed Apr 15 2020 Jan Palus <atler@pld-linux.org> 5e953ef
- up to 59

* Fri Mar 27 2020 Jan Palus <atler@pld-linux.org> 6a3ebac
- up to 58

* Sun Mar 03 2019 Jan Palus <atler@pld-linux.org> 17fbd67
- up to 57

* Fri Jan 25 2019 Arkadiusz Miśkiewicz <arekm@maven.pl> dbc54c4
- rel 2; search history in typed prefix range

* Sun Jan 14 2018 Jan Palus <atler@pld-linux.org> 1702196
- up to 56c

* Fri Sep 29 2017 Arkadiusz Miśkiewicz <arekm@maven.pl> 7297897
- up to 56b

* Sat Aug 12 2017 Jan Palus <atler@pld-linux.org> 7438aa9
- up to 56

* Mon Nov 14 2016 Jan Palus <atler@pld-linux.org> a9e5df8
- up to 54

* Tue Oct 25 2016 Jakub Bogusz <qboosh@pld-linux.org> a311ce5
- updated to 53a
- removed obsolete cmdline-length patch (16k is now default if not using MKSH_SMALL)

* Sat Mar 26 2016 Jan Palus <atler@pld-linux.org> a7f76f9
- up to 52c

* Thu Jan 21 2016 Jan Palus <atler@pld-linux.org> 8391f6f
- up to 52b

* Wed Dec 16 2015 Arkadiusz Miśkiewicz <arekm@maven.pl> f994223
- up to 52; upsteam dropped stop alias

* Thu Oct 01 2015 Arkadiusz Miśkiewicz <arekm@maven.pl> 098708b
- rel 3; back to 16k

* Thu Oct 01 2015 Arkadiusz Miśkiewicz <arekm@maven.pl> af8e7aa
- try 256k

